
local addonName, addon = ...
local QGT = LibStub("AceAddon-3.0")

local WidgetLists = AceGUIWidgetLSMlists

local floor = math.floor
local fmod = math.fmod
local format = string.format
local strsub = string.sub
local strlen = string.len

local QGTF = QGT.frame
local OTF = ObjectiveTrackerFrame
local OTFHeader = OTF.HeaderMenu

local db
local anchors = { "TOPLEFT", "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT" }
local strata = { "LOW", "MEDIUM", "HIGH" }
local flags = { "None", "Outline", "Monochrome Outline" }

local cNote = "|cff00ffe3"
local experimental = "|cffff7fff[experimental]|r"
local warning = "|cffff7f00Warning:|r UI will be reloaded!"
print(QGT.title.." |cffffffffv"..QGT.version.."|r");
local SetStrata, DecToHex, RgbToHex

local defaults = {
	profile = {
		anchorPoint = "TOPRIGHT",
		xOffset = -85,
		yOffset = -200,
		maxHeight = 400,
		frameStrata = "LOW",
		
		bgr = "Solid",
		bgrColor = { r=0, g=0, b=0, a=0.7 },
		border = "None",
		borderColor = { r=1, g=1, b=1 },
		classBorder = false,
		borderAlpha = 1,
		borderThickness = 16,
		bgrInset = 4,
		
		font = "Friz Quadrata TT",
		fontSize = 12,
		fontFlag = "None",
		fontShadow = true,
				
		hbColor = { r=1, g=0.82, b=0 },
		hbBorderColor = false,
		keyBindMinimize = "",
		hbOtherButtons = true,
		
		qiBgrBorder = false,
		qiXOffset = -5,
		qiActiveButton = true,
		
		blockTooltips = true,
		hideEmptyTracker = false,
		collapseInInstance = false,
		
		addonPetTracker = true,
		addonMasque = false,
		
		collapsed = false,
	}
}

local options = {
	name = QGT.title.." |cffffffffv"..QGT.version.."|r",
	type = "group",
	get = function(info) return db[info[#info]] end,
	args = {
		general = {
			name = "Options",
			type = "group",
			order = 1,
			args = {
				sec1 = {
					name = "Position  /  Size",
					type = "group",
					inline = true,
					order = 1,
					args = {
						anchorPoint = {
							name = "  Anchor point  ",
							type = "select",
							values = anchors,
							get = function()
								for k, v in pairs(anchors) do
									if db.anchorPoint == v then
										return k
									end
								end
							end,
							set = function(_, value)
								db.anchorPoint = anchors[value]
								db.xOffset = 0
								db.yOffset = 0
								QGT:MoveTracker()
							end,
							order = 1.1,
						},
						xOffset = {
							name = "X offset",
							type = "range",
							min = 0,
							max = 0,
							step = 1,
							set = function(_, value)
								db.xOffset = value
								QGT:MoveTracker()
							end,
							order = 1.2,
						},
						yOffset = {
							name = "Y offset",
							type = "range",
							min = 0,
							max = 0,
							step = 1,
							set = function(_, value)
								local mod = (db.anchorPoint == "LEFT" or db.anchorPoint == "RIGHT") and 2 or 1
								if (QGT.screenHeight/mod)-abs(value) < 100/mod then
									value = value < 0 and -(QGT.screenHeight - 100) / mod or (QGT.screenHeight - 100) / mod
								end
								db.yOffset = value
								QGT:MoveTracker()
								QGT:SetSize()
							end,
							order = 1.3,
						},
						spacerMid1 = {
							name = " ",
							width = "full",
							type = "description",
							order = 1.35,
						},
						maxHeight = {
							name = "Max. height",
							type = "range",
							min = 100,
							max = 100,
							step = 1,
							set = function(_, value)
								db.maxHeight = value
								QGT:SetSize()
							end,
							order = 1.4,
						},
						maxHeightNote = {
							name = cNote.."       Max. height is related with value Y offset.\n       Content is lesser ... height is automatically increases.\n       Content is greater ... enables scrolling.",
							width = "double",
							type = "description",
							order = 1.41,
						},
						spacerMid2 = {
							name = " ",
							width = "full",
							type = "description",
							order = 1.45,
						},
						frameStrata = {
							name = "  Strata  ",
							type = "select",
							values = strata,
							get = function()
								for k, v in pairs(strata) do
									if db.frameStrata == v then
										return k
									end
								end
							end,
							set = function(_, value)
								db.frameStrata = strata[value]
								SetStrata()
							end,
							order = 1.5,
						},
						spacerBtm = {
							name = " ",
							width = "full",
							type = "description",
							order = 1.6,
						},
					},
				},
				sec2 = {
					name = "Background  /  Border",
					type = "group",
					inline = true,
					order = 2,
					args = {
						bgr = {
							name = " Background texture ",
							type = "select",
							dialogControl = "LSM30_Background",
							values = WidgetLists.background,
							set = function(_, value)
								db.bgr = value
								QGT:SetBackground()
								QGT:MoveButtons()
							end,
							order = 2.1,
						},
						bgrColor = {
							name = "Background color",
							type = "color",
							hasAlpha = true,
							get = function()
								return db.bgrColor.r, db.bgrColor.g, db.bgrColor.b, db.bgrColor.a
							end,
							set = function(_, r, g, b, a)
								db.bgrColor.r = r
								db.bgrColor.g = g
								db.bgrColor.b = b
								db.bgrColor.a = a
								QGT:SetBackground()
							end,
							order = 2.2,
						},
						bgrNote = {
							name = cNote.."For a custom background texture set to the white color.",
							width = "normal",
							type = "description",
							order = 2.21,
						},
						spacerMid1 = {
							name = " ",
							width = "full",
							type = "description",
							order = 2.25,
						},
						border = {
							name = " Border texture ",
							type = "select",
							dialogControl = "LSM30_Border",
							values = WidgetLists.border,
							set = function(_, value)
								db.border = value
								QGT:SetBackground()
								QGT:MoveButtons()
							end,
							order = 2.3,
						},
						borderColor = {
							name = "Border color",
							type = "color",
							disabled = function()
								return db.classBorder
							end,
							get = function()
								return db.borderColor.r, db.borderColor.g, db.borderColor.b
							end,
							set = function(_, r, g, b)
								db.borderColor.r = r
								db.borderColor.g = g
								db.borderColor.b = b
								QGT:SetBackground()
							end,
							order = 2.4,
						},
						classBorder = {
							name = "Border color by |cff%sClass|r",
							type = "toggle",
							set = function()
								db.classBorder = not db.classBorder
								QGT:SetBackground()
							end,
							order = 2.5,
						},
						spacerMid2 = {
							name = " ",
							width = "full",
							type = "description",
							order = 2.55,
						},
						borderAlpha = {
							name = "Border transparency",
							type = "range",
							min = 0,
							max = 1,
							step = 0.05,
							set = function(_, value) 
								db.borderAlpha = value
								QGT:SetBackground()
							end,
							order = 2.6,
						},
						borderThickness = {
							name = "Border thickness",
							type = "range",
							min = 1,
							max = 24,
							step = 0.5,
							set = function(_, value) 
								db.borderThickness = value
								QGT:SetBackground()
							end,
							order = 2.7,
						},
						bgrInset = {
							name = "Background inset",
							type = "range",
							min = 0,
							max = 10,
							step = 0.5,
							set = function(_, value) 
								db.bgrInset = value
								QGT:SetBackground()
								QGT:MoveButtons()																								
							end,
							order = 2.8,
						},
						spacerBtm = {
							name = " ",
							width = "full",
							type = "description",
							order = 2.9,
						},
					},
				},
				sec3 = {
					name = "Text",
					type = "group",
					inline = true,
					order = 3,
					args = {
						font = {
							name = "  Font  ",
							type = "select",
							dialogControl = "LSM30_Font",
							values = WidgetLists.font,
							set = function(_, value)
								db.font = value
								QGT:SetText()
								ObjectiveTracker_Update()
							end,
							order = 3.1,
						},
						fontSize = {
							name = "Font size",
							type = "range",
							min = 8,
							max = 24,
							step = 1,
							set = function(_, value) 
								db.fontSize = value
								QGT:SetText()
								ObjectiveTracker_Update()
							end,
							order = 3.2,
						},
						fontFlag = {
							name = "  Font flag  ",
							type = "select",
							values = flags,
							get = function()
								for k, v in pairs(flags) do
									if db.fontFlag == v then
										return k
									end
								end
							end,
							set = function(_, value)
								db.fontFlag = flags[value]
								QGT:SetText()
								ObjectiveTracker_Update()
							end,
							order = 3.3,
						},
						spacerMid = {
							name = " ",
							width = "full",
							type = "description",
							order = 3.35,
						},
						fontShadow = {
							name = "Font shadow",
							type = "toggle",
							set = function()
								db.fontShadow = not db.fontShadow
								QGT:SetText()
								ObjectiveTracker_Update()
							end,
							order = 3.4,
						},
						spacerBtm = {
							name = " ",
							width = "full",
							type = "description",
							order = 3.5,
						},
					},
				},
				sec4 = {
					name = "Header buttons",
					type = "group",
					inline = true,
					order = 4,
					args = {
						hbColor = {
							name = "Buttons color",
							type = "color",
							disabled = function()
								return db.hbBorderColor
							end,
							get = function()
								return db.hbColor.r, db.hbColor.g, db.hbColor.b
							end,
							set = function(_, r, g, b)
								db.hbColor.r = r
								db.hbColor.g = g
								db.hbColor.b = b
								QGT:SetBackground()
							end,
							order = 4.1,
						},
						hbBorderColor = {
							name = "Share border color",
							desc = "The color of all header buttons is shared with the border color.",
							type = "toggle",
							set = function()
								db.hbBorderColor = not db.hbBorderColor
								QGT:SetBackground()								
							end,
							order = 4.2,
						},
						keyBindMinimize = {
							name = "Key - Minimize button",
							type = "keybinding",
							set = function(_, value)
								SetOverrideBinding(QGTF, false, db.keyBindMinimize, nil)
								if value ~= "" then
									local key = GetBindingKey("EXTRAACTIONBUTTON1")
									if key == value then
										SetBinding(key)
										SaveBindings(GetCurrentBindingSet())
									end
									SetOverrideBindingClick(QGTF, false, value, OTFHeader.MinimizeButton:GetName())
								end
								db.keyBindMinimize = value
							end,
							order = 4.3,
						},
						spacerMid = {
							name = " ",
							width = "full",
							type = "description",
							order = 4.35,
						},
						hbOtherButtons = {
							name = "Show other buttons",
							desc = "Show Quest Log and Achievements buttons in header next to Minimize button.",
							type = "toggle",
							confirm = true,
							confirmText = warning,
							set = function()
								db.hbOtherButtons = not db.hbOtherButtons
								ReloadUI()
							end,
							order = 4.4,
						},
						spacerBtm = {
							name = " ",
							width = "full",
							type = "description",
							order = 4.5,
						},
					},
				},
				sec5 = {
					name = "Quest item buttons",
					type = "group",
					inline = true,
					order = 5,
					args = {
						qiBgrBorder = {
							name = "Show buttons block background and border",
							width = "double",
							type = "toggle",
							set = function()
								db.qiBgrBorder = not db.qiBgrBorder
								QGT:SetBackground()
								QGT:MoveButtons()
							end,
							order = 5.1,
						},
						qiXOffset = {
							name = "X offset",
							type = "range",
							min = -10,
							max = 10,
							step = 1,
							set = function(_, value)
								db.qiXOffset = value
								QGT:MoveButtons()
							end,
							order = 5.2,
						},
						qiActiveButton = {
							name = "Enable Active button",
							type = "toggle",
							set = function()
								db.qiActiveButton = not db.qiActiveButton
								if db.qiActiveButton then
									QGT:UpdateActiveButton()
								else
									local bar = ExtraActionBarFrame
									bar.button:SetAlpha(1)
									if not HasExtraActionBar() then
										bar.intro:Stop()
										bar:SetAlpha(0)
										bar:Hide()
									end
									QGTF.ActiveButton:Hide()
									ClearOverrideBindings(QGTF.ActiveButton)
								end
							end,
							order = 5.3,
						},
						keyBindActiveButton = {
							name = "Key - Active button",
							type = "keybinding",
							disabled = function()
								return not db.qiActiveButton
							end,
							get = function()
								local key = GetBindingKey("EXTRAACTIONBUTTON1")
								return key
							end,
							set = function(_, value)
								local key = GetBindingKey("EXTRAACTIONBUTTON1")
								if key then
									SetBinding(key)
								end
								if value ~= "" then
									SetBinding(value, "EXTRAACTIONBUTTON1")
								end
								SaveBindings(GetCurrentBindingSet())
							end,
							order = 5.4,
						},
						keyBindActiveButtonNote = {
							name = cNote.."      Key bind is shared with\n      the EXTRAACTIONBUTTON1.",
							width = "normal",
							type = "description",
							order = 5.41,
						},
						qiActiveButtonDesc = {
							name = "         "..experimental.." Show Quest item button for CLOSEST quest as \"Extra Action Button\".\n",
							width = "full",
							type = "description",
							order = 5.5,
						},
					},
				},
				sec6 = {
					name = "Other options",
					type = "group",
					inline = true,
					order = 6,
					args = {
						spacerTop = {
							name = " ",
							width = "full",
							type = "description",
							order = 6.1,
						},
						blockTooltips = {
							name = "Show block tooltips",
							desc = "Show Quest and Achievement tooltips.",
							type = "toggle",
							set = function()
								db.blockTooltips = not db.blockTooltips
							end,
							order = 6.2,
						},
						hideEmptyTracker = {
							name = "Hide empty tracker",
							desc = "Hide the tracker when no quests or achievements are being tracked.",
							type = "toggle",
							set = function()
								db.hideEmptyTracker = not db.hideEmptyTracker
								QGT:ToggleEmptyTracker()
							end,
							order = 6.3,
						},
						collapseInInstance = {
							name = "Collapse in instance",
							desc = "Automatically collapses the tracker when entering an instance.",
							type = "toggle",
							set = function()
								db.collapseInInstance = not db.collapseInInstance
								if db.collapseInInstance then
									QGTF:RegisterEvent("PLAYER_ENTERING_WORLD")
								else
									QGTF:UnregisterEvent("PLAYER_ENTERING_WORLD")
								end
							end,
							order = 6.4,
						},
						spacerBtm = {
							name = " ",
							width = "full",
							type = "description",
							order = 6.5,
						},
					},
				},
				sec7 = {
					name = "Other options",
					type = "group",
					inline = true,
					order = 7,
					args = {
						spacerBtm = {
							name = " ",
							width = "full",
							type = "description",
							order = 7.4,
						},
					},
				},
			},
		},
	},
}

--function QGT:CheckAddOnCompat(name, version)
--	local ver = "---"
--	if IsAddOnLoaded(name) then
--		local actualVersion = GetAddOnMetadata(name, "Version")
--		ver = ("|cff%s"..actualVersion.."|r"):format(actualVersion == version and "00d200" or "ff0000")
--	end
--	local options = options.args.general.args.sec7.args
--	options["addon"..name].desc = (options["addon"..name].desc):format(ver)
--end

function QGT:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory(QGT.optionsFrames.profiles)
	InterfaceOptionsFrame_OpenToCategory(QGT.optionsFrames.profiles)
	InterfaceOptionsFrame_OpenToCategory(QGT.optionsFrames.general)
end

function QGT:InitProfile(event, database, profile)
	db = self.db.profile
	
	ClearOverrideBindings(QGTF)
	ObjectiveTracker_Expand()
	
	SetStrata()
	self:Init()
end

function QGT:SetupOptions()
	self.db = LibStub("AceDB-3.0"):New(addonName.."DB", defaults, true)
	self.options = options
	db = self.db.profile
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	options.args.profiles.args.reset.confirm = true
	options.args.profiles.args.copyfrom.confirm = true
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options, nil)
	
	options.args.general.args.sec2.args.classBorder.name = options.args.general.args.sec2.args.classBorder.name:format(RgbToHex(self.classColor))
	
	self.optionsFrames = {}
	self.optionsFrames.general = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, self.title, nil, "general")
	self.optionsFrames.profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, "Profiles", self.title, "profiles")
	
	LibStub("AceConsole-3.0"):RegisterChatCommand("kt", function() self:OpenOptions() end)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "InitProfile")
	self.db.RegisterCallback(self, "OnProfileCopied", "InitProfile")
	self.db.RegisterCallback(self, "OnProfileReset", "InitProfile")
end

function SetStrata() 
	QGTF:SetFrameStrata(db.frameStrata)
	QGTF.Buttons:SetFrameStrata(db.frameStrata)
end

function DecToHex(num)
	local b, k, hex, d = 16, "0123456789abcdef", ""
	while num > 0 do
		d = fmod(num, b) + 1
		hex = strsub(k, d, d)..hex
		num = floor(num/b)
	end
	hex = hex == "" and "0" or hex
	return hex
end
function RgbToHex(color)
	local r, g, b = DecToHex(color.r*255), DecToHex(color.g*255), DecToHex(color.b*255)
	r = strlen(r) < 2 and "0"..r or r
	g = strlen(g) < 2 and "0"..g or g
	b = strlen(b) < 2 and "0"..b or b
	return r..g..b
end