﻿local addonName, at = ...
if not at.QuestGuru then return end  -- "YOU SHALL NOT PASS!" - Gandolf
local QuestGuru = at.QuestGuru
local main = CreateFrame("Frame", "QuestGuruConfig")
main.name = "QuestGuru"

	local Profile_OnChange = {}
	
--optional
--[[
main.okay = function() end
main.cancel = function() end
main.refresh = function() end--]]
main.default = function()
	local guid = QuestGuru.PLAYER_GUID
	
	QuestGuruDB = QuestGuruDB:CopyTable(QuestGuru.defaults)
	
	for x, v in pairs(Profile_OnChange) do
		v(x)
	end
	ReloadUI()
end

local tooltip = CreateFrame("GameTooltip", addonName .. "Tooltip", nil, "GameTooltipTemplate")

QuestGuru:AddCallback('VariablesLoaded', 'options', function(self)
	local OptionsScrollFrame = CreateFrame('ScrollFrame', gsub(addonName, " ", "") .. 'OptionsScrollFrame', main, 'UIPanelScrollFrameTemplate')
	OptionsScrollFrame:SetPoint('TOPLEFT', main, 'TOPLEFT', 10, -32)
	OptionsScrollFrame:SetPoint('BOTTOMRIGHT', main, 'BOTTOMRIGHT', -30, 10)
	
	local OptionsScrollChild = CreateFrame('Frame', '$parentChild', OptionsScrollFrame)
	OptionsScrollChild:SetPoint('TOPLEFT', OptionsScrollFrame, 'TOPLEFT')
	OptionsScrollChild:SetSize(580, 750)
	OptionsScrollFrame:SetScrollChild(OptionsScrollChild)


	local Title = main:CreateFontString( nil, "ARTWORK", "GameFontNormalLarge" )
	Title:SetPoint( "TOP", 0, -10 )
	Title:SetText( addonName )

	
	local GameTooltip = GameTooltip
	local function Frame_OnEnter(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		local tooltip = self.tooltip
		for x = 1, #tooltip do
			GameTooltip:AddLine(tooltip[x], 1, 1, 1)
		end
		GameTooltip:Show()
	end
	local function Frame_OnLeave(self)
		GameTooltip:Hide()
	end
	
	local prev
	local function CreateContainer( name, func )
		local Container = CreateFrame( "Frame", "$parent".. gsub(name, " ", ""), OptionsScrollChild, "OptionsBoxTemplate" )
		Container:SetPoint("LEFT", OptionsScrollChild, 10, 0)
		Container:SetPoint("RIGHT", OptionsScrollChild, -10, 0)
		if prev then
			Container:SetPoint("TOP", prev, "BOTTOM", 0, -8)
		else
			Container:SetPoint("TOP", OptionsScrollChild, "TOP", 0, -8)
		end
		
		local title = Container:CreateFontString( nil, "ARTWORK", "GameFontNormalLarge" )
		title:SetPoint( 'CENTER', Container, 'TOP', 0, 0 )
		title:SetText( name )
		
		local Background = Container:CreateTexture( nil, "BACKGROUND" )
		Background:SetTexture(0, 0, 0, 0.15)
		Background:SetPoint("TOPLEFT", 5, -5)
		Background:SetPoint("BOTTOMRIGHT", -5, 5)
		
		func( Container, name )
		
		prev = Container
		return Container
	end
	local function Slider_OnValueChanged(self, num)
		QuestGuru.db[self.category][self.setting] = num
		self.text:SetText( num )
		JamPlatesAccessoriesDB[JamPlatesAccessoriesCP][self.category][self.setting] = num
	end
	local function CreateSlider(name, parent, category, setting, tooltip, width, height)
		local Slider = CreateFrame( "Slider", "$parent".. gsub(name, " ", ""), parent, "OptionsSliderTemplate" );
		Slider:SetMinMaxValues( 0, 2 )
		Slider:SetValue( QuestGuru.db[category][setting] )
		Slider:HookScript( "OnValueChanged", Slider_OnValueChanged )
		local Text = _G[ Slider:GetName().."Text" ]
		Text:ClearAllPoints();
		Text:SetPoint( "TOP", Slider, "BOTTOM", 0, 0 )
		Text:SetText( QuestGuru.db[category][setting] )
		Slider.text = Text
		Slider.category = category
		Slider.setting = setting
		
		if tooltip then
			Slider.tooltip = tooltip
			Slider:HookScript('OnEnter', Frame_OnEnter)
			Slider:HookScript('OnLeave', Frame_OnLeave)
		end
		
		local Label = parent:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
		Label:SetText( name .. " : " )
		Label.Slider = Slider
		
		Profile_OnChange[#Profile_OnChange +1] = function()
			Slider:SetValue( QuestGuru.db[Slider.category][Slider.setting] )
			Text:SetText( QuestGuru.db[Slider.category][Slider.setting] )
		end
		
		Slider:SetPoint('LEFT', Label, 'RIGHT', 11, 0)
		Slider:SetPoint('RIGHT', parent, 'RIGHT', -11, 0)
		return Label
	end

	
	local totalHeight = 0
	CreateContainer('Scales', function(self)
		local HappyScale = CreateSlider('Happiness Scale', self, 'happiness', 'scale', {"Increase or decrease the rate happiness is gained."})
		HappyScale:SetPoint('TOPLEFT', self, 'TOPLEFT', 10, -16)
		
		local LoyaltyScale = CreateSlider('Loyalty Scale', self, 'loyalty', 'scale', {"Increase or decrease the rate loyalty is gained."})
		LoyaltyScale:SetPoint('TOPLEFT', HappyScale, 'BOTTOMLEFT', 10, -16)
		
		self:SetHeight(90)
		totalHeight = totalHeight + 90
	end)
	
	
	OptionsScrollChild:SetHeight(totalHeight)
end)


InterfaceOptions_AddCategory( main )
_G['SLASH_' .. addonName .. 1] = "/petship"
SlashCmdList[addonName] = function(msg)
	if InCombatLockdown() then return print('Commands disabled in combat.') end
	if msg == 'reset' then
		PetshipDB = nil
		PetshipSetting = nil
		QuestGuru:UnregisterEvent('PLAYER_LOGOUT')
		ReloadUI()
	else
		InterfaceOptionsFrame_Show() -- redundant, yes; safer, also yes.
		InterfaceOptionsFrame_OpenToCategory(main.name)
	end
end
