
-------------------
-- Locals
-------------------

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local name, QuestGuru = ...

-------------------
-- Media
-------------------

local LSM = LibStub("LibSharedMedia-3.0")

LSM:Register("sound", "Alarm Clock 1",    "Sound\\Interface\\AlarmClockWarning1.wav")
LSM:Register("sound", "Alarm Clock 2",    "Sound\\Interface\\AlarmClockWarning2.wav")
LSM:Register("sound", "Alarm Clock 3",    "Sound\\Interface\\AlarmClockWarning3.wav")
LSM:Register("sound", "Bell - Alliance",  "Sound\\Doodad\\BellTollAlliance.wav")
LSM:Register("sound", "Bell - Horde",     "Sound\\Doodad\\BellTollHorde.wav")
LSM:Register("sound", "Bell - Night Elf", "Sound\\Doodad\\BellTollNightElf.wav")
LSM:Register("sound", "Drum Hit",         "Sound\\Doodad\\BellTollTribal.wav")
LSM:Register("sound", "Gong - Troll",     "Sound\\Doodad\\G_GongTroll01.wav")
LSM:Register("sound", "Karazhan Bell",    "Sound\\Doodad\\KharazahnBellToll.wav")
LSM:Register("sound", "Mellow Bells",     "Sound\\Spells\\ShaysBell.wav")
LSM:Register("sound", "Level Up2",		"Sound/Interface/levelup2.ogg")
LSM:Register("sound", "Auction Window Close",		"Sound/Interface/AuctionWindowClose.ogg")
LSM:Register("sound", "Auction Window Open",		"Sound/Interface/AuctionWindowOpen.ogg")
LSM:Register("sound", "Peasent Ready",		"Sound/Creature/Peasant/PeasantReady1.ogg")
LSM:Register("sound", "Peasent Yes1",		"Sound/Creature/Peasant/PeasantYes1.ogg")
LSM:Register("sound", "Peasent Yes4",		"Sound/Creature/Peasant/PeasantYes4.ogg")
LSM:Register("sound", "Peasent Done",		"Interface\\AddOns\\QuestProgressSound\\Sounds\\Peasant_work_done.ogg")
LSM:Register("sound", "Peon Building Complete",		"Sound/Creature/Peon/PeonBuildingComplete1.ogg")
LSM:Register("sound", "Peon Ready",		"Sound/Creature/Peon/PeonReady1.ogg")
LSM:Register("sound", "Peon Yes",		"Sound/Creature/Peon/PeonYes3.ogg")
LSM:Register("sound", "Peon What",		"Sound\\Creature\\Peon\\PeonWhat4.ogg")
LSM:Register("sound", "Ding",		"Interface\\AddOns\\QuestProgressSound\\Sounds\\ding.ogg")
LSM:Register("sound", "Tada",		"Interface\\AddOns\\QuestProgressSound\\Sounds\\tada.ogg")
LSM:Register("sound", "Info",		"Interface\\AddOns\\QuestProgressSound\\Sounds\\info.ogg")
LSM:Register("sound", "Alert",		"Interface\\AddOns\\QuestProgressSound\\Sounds\\Alert.ogg")

function QuestGuru:PLAYER_LOGIN()
	db = LibStub("AceDB-3.0"):New("QuestGuruDB", { profile = {
		ShowLevels = true, ShowTooltips = false, soundQuestComplete = "Alarm Clock 3", 
		soundObjectiveComplete = "Alarm Clock 3",soundObjectiveUpdate = "Alarm Clock 3",
	}}, "Default").profile


	AceConfig:RegisterOptionsTable(name, QuestGuru.CreateConfig)
	AceConfigDialog:AddToBlizOptions(name, "QuestGuru")
	AceConfigDialog:SetDefaultSize(name, 600, 300)

	_G.SlashCmdList["QUESTGURU"] = function() AceConfigDialog:Open(name) end
	_G["SLASH_QUESTGURU1"] = "/questguru"
	_G["SLASH_QUESTGURU2"] = "/qg"
	QuestGuru:CreateConfig()
	self:UnregisterEvent("PLAYER_LOGIN")
end


local appName = ...
local A, L, DF = unpack(select(2, ...))

local AC = LibStub("AceConfig-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local LSM = LibStub("LibSharedMedia-3.0")

local DEFAULT_WIDTH = 610
local DEFAULT_HEIGHT = 450

ACD:SetDefaultSize(DEFAULT_WIDTH, DEFAULT_HEIGHT, appName)

local format, math = format, math

-- GLOBALS: InterfaceOptionsFrame_OpenToCategory
-- GLOBALS: InCombatLockdown, LowHealthAlarm_DB

function A:ToggleOptions()
	if InCombatLockdown() then
		self:PrintErr("CHAT/CombatLoadError")
		return
	end
	InterfaceOptionsFrame_OpenToCategory("QuestGuru")
end

function A:SetDefaultOptions()
	LowHealthAlarm_DB = DF
	ACR:NotifyChange(appName)
end

function A:BuildOptionsTable()
	AC:RegisterOptionsTable(appName, self.Options)
	self.optionsFrame = ACD:AddToBlizOptions(appName, "QuestGuru")
	self.optionsFrame.default = function() self:SetDefaultOptions() end

	-- Settings here need to be the same name as the settings in DF...
	self.Options.args = {
		VersionHeader = {
			name = format("DLG/Version", self:GetVersion(false, false)),
			type = "header",
			order = 0
		},
		Enabled = {
			name = "DLG/Enable",
			desc = "DLGTT/Enable",
			type = "toggle",
			order = 10,
			set = function(info, value)
				if value == false then
					A:Disable()
				else
					A:Enable()
				end
				QuestGuruDB.Enabled = value
			end
		},
		LoginMessage = {
			name = "DLG/Login Message",
			desc = "DLGTT/Login Message",
			type = "toggle",
			order = 20,
		},
		Settings = {
			name = "QuestGuru/Main Config",
			type = "group",
			order = 30,
			inline = true,
			args = {
				Thresholds = {
					name = "DLG/Threshold Settings",
					--desc = "The settings controlling when the addon will make noise",
					type = "group",
					inline = true,
					order = 20,
					args = {
						HP_CRITICAL = {
							name = "DLG/\"Critical\" Health",
							desc = "DLGTT/\"Critical\" Health",
							type = "range",
							min = 0.01,
							max = self:GetSetting("HP_WARNING") - 0.01, -- Temporary... Will be modified by setters/getters here.
							step = 0.01,
							isPercent = true,
							set = function(info, value)
									-- Sets the min/max of neighbouring sliders so values can't overlap and break the addon.
									self.Options.args.Settings.args.Thresholds.args.HP_WARNING.min = value + 0.01
									QuestGuruDB[info[#info]] = value
								end,
							order = 0
						},
						HP_WARNING  = {
							name = "DLG/\"Dangerous\" Health",
							desc = "DLGTT/\"Dangerous\" Health",
							type = "range",
							min = self:GetSetting("HP_CRITICAL") + 0.01,
							max = self:GetSetting("HP_LOW") - 0.01,
							step = 0.01,
							isPercent = true,
							set = function(info, value)
									-- Sets the min/max of neighbouring sliders so values can't overlap and break the addon.
									self.Options.args.Settings.args.Thresholds.args.HP_CRITICAL.max = value - 0.01
									self.Options.args.Settings.args.Thresholds.args.HP_LOW.min = value + 0.01
									QuestGuruDB[info[#info]] = value
								end,
							order = 10
						},
						HP_LOW      = {
							name = "DLG/\"Low\" Health",
							desc = "DLGTT/\"Low\" Health",
							type = "range",
							min = self:GetSetting("HP_WARNING") + 0.01,
							max = 0.8,
							step = 0.01,
							isPercent = true,
							set = function(info, value)
									-- Sets the min/max of neighbouring sliders so values can't overlap and break the addon.
									self.Options.args.Settings.args.Thresholds.args.HP_WARNING.max = value - 0.01
									QuestGuruDB[info[#info]] = value
								end,
							order = 20
						}
					}
				},
				Speed = {
					name = "DLG/Speed Settings",
					--desc = "How often the addon will make noise",
					type = "group",
					inline = true,
					order = 30,
					args = {
						ExplanationText = {
							type = "description",
							name = "DLG/Speed Explanation",
							width = "full",
							order = 0
						},
						Base = {
							name = "DLG/Base Speed",
							desc = "DLGTT/Base Speed",
							type = "range",
							min = 0.1,
							max = math.huge,
							softMax = 4,
							bigStep = 0.1,
							order = 1
						},
						Multiplier = {
							name = "DLG/Multiplier",
							desc = "DLGTT/Multiplier",
							type = "range",
							min = 0.1,
							max = math.huge,
							softMin = 1,
							softMax = 4,
							bigStep = 0.5,
							order = 2
						}
					}
				},
				Testing = {
					name = "DLG/Test Buttons",
					type = "group",
					inline = true,
					order = 40,
					args = {
						SlowBeep = {
							name = "DLG/\"Low\" Beep",
							type = "execute",
							order = 0,
							func = function()
							  A:PrintMessage(format("CHAT/Testing", "DLG/\"Low\" Health"))
							  A:SetBeepSpeed(self:GetSetting("Base") * self:GetSetting("Multiplier"))
							end
						},
						MedBeep = {
							name = "DLG/\"Dangerous\" Beep",
							type = "execute",
							order = 10,
							func = function()
							  A:PrintMessage(format("CHAT/Testing", "DLG/\"Dangerous\" Health"))
							  A:SetBeepSpeed(self:GetSetting("Base"))
							end
						},
						FastBeep = {
							name = "DLG/\"Critical\" Beep",
							type = "execute",
							order = 20,
							func = function()
							  A:PrintMessage(format("CHAT/Testing", "DLG/\"Critical\" Health"))
							  A:SetBeepSpeed(self:GetSetting("Base") / self:GetSetting("Multiplier"))
							end
						},
						StopBeep = {
							name = "DLG/Stop All",
							type = "execute",
							order = 30,
							width = "full",
							func = function()
								A:PrintMessage("CHAT/StopBeeps")
								A:SetBeepSpeed(0)
							end
						}
					}
				},
				Misc = {
					name = "DLG/Misc Settings",
					type = "group",
					order = 50,
					inline = true,
					args = {
						SkipFirstBeep = {
							name = "DLG/Skip First Beep",
							desc = "DLGTT/Skip First Beep",
							type = "toggle"
						}
					}
				},
				BeepFile = {
					type = "select",
					dialogControl = "LSM30_Sound",
					name = "DLG/Sound Effect",
					desc = "DLGTT/Sound Effect",
					values = LSM:HashTable("sound"),
					order = 0,
				},
				BeepVolume = {
					name = "DLG/Volume",
					desc = "DLGTT/Volume",
					type = "range",
					min = 1,
					max = 5,
					step = 1,
					order = 10
				},
				SoundChannel = {
					name = "DLG/Channel",
					desc = "DLGTT/Channel",
					type = "select",
					style = "dropdown",
					order = 9,
					values = {
						["Master"] = "Master",
						["SFX"] = "Sound Effects",
						["Music"] = "Music",
						["Ambience"] = "Ambience",
						["Dialog"] = "Dialog"
					}
				}
			}
		}
	}
end



